part of tecfy_core_client_package;

class PropertyDropdown extends Property {
  late String entityName;
  late String dbNameEntity;
  Property? entityDisplayProp;

  /// <summary>
  /// used to get the list using custom EndPoint like "Item/GetItemInfo?brand={brandId}&model={modelId}&category={itemCategoryId}&year={year}"
  /// </summary>
  late String dropdownUrl;

  late bool isFilter;

  /// <summary>
  /// where value of FilterField is null of embty then the value of FilterValueField must refer to Array proerty of same dropdwn Entity
  /// </summary>
  late String filterField;
  late String filterQuery;

  /// <summary>
  /// where value of FilterField is null of embty then the value of FilterValueField must refer to Array proerty of same dropdwn Entity
  /// </summary>
  late String filterValueField;

  List<Map<String, String>>? options;

  /// <summary>
  /// Fileds that in main entity and dropdown entity with the same name, that you need to update the value of the main entity with the value in dropdown entity
  /// For example: when select item the UOM_ID of the invoice will be overwrten
  /// Can add multi properties seperated by comma
  /// </summary>
  Map<String, String>? overwritePropertiesOnSelect;

  PropertyDropdown(
      {required String dbName,
      required String nameAr,
      required String nameEn,
      bool isRequired = false,
      bool readOnly = false,
      required this.entityName,
      this.overwritePropertiesOnSelect,
      this.isFilter = false,
      this.filterField = '',
      this.filterQuery = '',
      this.filterValueField = ''})
      : super(
            dbName: dbName,
            nameAr: nameAr,
            nameEn: nameEn,
            readOnly: readOnly,
            propertyType: PropertyTypes.Dropdown,
            valueType: ValueTypes.Text,
            isRequired: isRequired) {
    dbNameEntity = this.dbName.length > 2
        ? this.dbName.substring(0, this.dbName.length - 2)
        : '';
    dropdownUrl = '';
    bindOptions();
  }

  PropertyDropdown.fromMap(Map<String, dynamic> row) : super.fromMap(row) {
    entityName = row['entityName'] ?? '';
    dropdownUrl = row['dropdownUrl'] ?? '';
    isFilter = row['isFilter'];
    filterField = lowerFirst(row['filterField'] ?? '') ?? '';
    filterQuery = row['filterQuery'] ?? '';

    overwritePropertiesOnSelect = {};

    if (row['overwritePropertiesOnSelect'] != null) {
      for (var k in row['overwritePropertiesOnSelect'].toString().split(',')) {
        var s = k.split(':');
        overwritePropertiesOnSelect![lowerFirst(s.first)!] =
            lowerFirst(s.last)!;
      }
    }

    dbName = lowerFirst(row['dbName']) ?? '';
    dbNameEntity = this.dbName.length > 2
        ? this.dbName.substring(0, this.dbName.length - 2)
        : '';
    filterValueField = lowerFirst(row['filterValueField'] ?? '') ?? '';
    if (entityName.isEmpty && filterValueField.isNotEmpty) {
      filterValueField = row['filterValueField'];
    }
    bindOptions();
  }

  void bindOptions() {
    if (entityName.isEmpty && filterValueField.isNotEmpty) {
      options = filterValueField.split(',').map<Map<String, String>>((e) {
        var s = e.split(':');
        return {
          'id': s.first,
          'nameEn': s.first,
          'nameAr': s.last,
        };
      }).toList();
    }
  }

  @override
  OperationResult isValid(data) {
    if (!super.isValid(data).success) return super.isValid(data);

    return OperationResult(success: true);
  }

  /// get the value and bind tp display widget
  Widget getDisplayWidget(Map<String, dynamic> data,
      {double size = 12, Color color = Colors.black}) {
    // if (this.dbNameEntity.isEmpty &&
    //     this.dbName.isNotEmpty &&
    //     this.dbName.length > 2) {
    //   dbNameEntity = this.dbName.substring(0, this.dbName.length - 2);
    // }
    // if (entityDisplayProp == null && entityName.isNotEmpty) {
    //   entityDisplayProp = EntityService.getEntity(entityName)?.displayField;
    // }
    // return Text(
    //     data[this.dbNameEntity] is String
    //         ? data[this.dbNameEntity]
    //         : data[this.dbNameEntity] == null
    //             ? '-'
    //             : '${data[this.dbNameEntity][entityDisplayProp?.dbName ?? 'name']}',
    //     style: TextStyle(fontSize: size, color: color));
    return Text(getDisplayText(data),
        style: TextStyle(fontSize: size, color: color));
  }

  /// get the value in text
  String getDisplayText(Map<String, dynamic> data) {
    if (entityName.isEmpty && filterValueField.isNotEmpty) {
      return data[dbName] ?? '';
    }
    if (this.dbNameEntity.isEmpty &&
        this.dbName.isNotEmpty &&
        this.dbName.length > 2) {
      dbNameEntity = this.dbName.substring(0, this.dbName.length - 2);
    }
    if (entityDisplayProp == null && entityName.isNotEmpty) {
      entityDisplayProp = EntityService.getEntity(entityName)?.displayField;
    }
    return data[this.dbNameEntity] is String
        ? data[this.dbNameEntity]
        : data[this.dbNameEntity] == null
            ? '-'
            : '${data[this.dbNameEntity][entityDisplayProp?.dbName ?? 'name']}';
  }

  Future<bool> getDefaultValue(Map<String, dynamic> data) async {
    //if(data[dbName] != null) return false;
    if (entityName.isEmpty && filterValueField.isNotEmpty) {
      data[dbName] = data[dbName] ?? defaultString;
      return true;
    }
    var dbNameEntity = dbName.substring(0, dbName.indexOf('Id'));
    var dbNameParent = dbName + '______Parent';
    try {
      //print('==================== set default value of dropdown =============== $dbName   $dbNameEntity');
      // Get the default value and set it
      if (defaultString?.isNotEmpty ?? false) {
        if (int.tryParse(defaultString ?? '') != null) {
          // TODO: fix after you make Id string
          if (data[dbName] == null || data[dbName] == 0) {
            data[dbName] = int.parse(defaultString ?? '');
            return true;
          }
        } else {
          if (defaultString?.contains('.') ?? false) {
            var parentPropName = defaultString?.split('.')[0] ?? '';
            var childPropName = defaultString?.split('.')[1] ?? '';

            var parentData = data[lowerFirst(parentPropName)];
            if (data[dbNameParent] != null &&
                data[dbNameParent]['id'] == parentData['id']) return false;
            data[dbNameParent] = parentData;
            //print('------ Parent Data $parentPropName $parentData');
            var childData = parentData[lowerFirst(childPropName)];
            //print('------ Child Data $childPropName $childData' );
            if (childData is int) {
              if (data[dbName] != childData) {
                data[dbName] = childData;
              }
            } else {
              // TODO: hadel this case
            }
          } else {
            // TODO: hadel this case
          }
        }

        if (data[dbName] != null &&
            data[dbName] != 0 &&
            (data[dbNameEntity] == null ||
                data[dbNameEntity]["id"] != data[dbName])) {
          print(
              '-------- get data for value ${data[dbName]} ${data[dbNameEntity] == null ? 'null' : data[dbNameEntity]['id']}');
          var result = await EntityService.getOne(entityName, data[dbName],
              withRelatedData: true);
          data[dbNameEntity] = result;
          if (data[dbNameEntity] != null) return true;
        }
      }
    } catch (err, t) {
      print(
          'XXXXXXXXXXXXxxxxxxxxxxxx ERROR  getDefaultValue for Dropdown $dbName: $err');
      print(t);
    }
    return false;
  }
}
