part of tecfy_core_client_package;

class DateTimeProperty extends StatefulWidget {
  final String? label;
  final String? description;
  final bool isRequired;
  final dynamic initalValue;
  final void Function(DateTime?)? onChanged;
  DateTimeProperty({
    this.label,
    this.description,
    this.isRequired = false,
    this.initalValue,
    this.onChanged,
  });
  @override
  _DateTimePropertyState createState() => _DateTimePropertyState();
}

class _DateTimePropertyState extends State<DateTimeProperty> {
  DateTime? _selectedDate;
  TimeOfDay? _selectedTime;

  DateTime? get _selectedDateTime {
    if (_selectedDate == null && _selectedTime == null) return null;
    if (_selectedTime == null) return _selectedDate!.toUtc();
    var d = _selectedDate;
    if (d == null) d = DateTime.now();
    d = d.add(
        Duration(hours: _selectedTime!.hour, minutes: _selectedTime!.minute));
    return d;
  }

  @override
  void initState() {
    print('initalValue : ${widget.initalValue}');
    super.initState();
  }

  void _onPressed() async {
    var value = await showDatePicker(
        builder: (context, child) {
          return Theme(
            data: Theme.of(context).copyWith(
              colorScheme: ColorScheme.light(
                primary: ThemeService.mainColor,
                onPrimary: Colors.white,
                onSurface: Colors.black,
              ),
              textButtonTheme: TextButtonThemeData(
                style: TextButton.styleFrom(
                  primary: Colors.black,
                ),
              ),
            ),
            child: child ?? Container(),
          );
        },
        context: context,
        firstDate: DateTime(1900),
        initialDate: _selectedDate ?? DateTime.now().toUtc(),
        lastDate: DateTime(2500));
    if (value == null) return;
    setState(() {
      _selectedDate = value.toUtc();
    });
    widget.onChanged?.call(_selectedDateTime);
  }

  void _onPressedTime() async {
    var value = await showTimePicker(
      builder: (context, child) {
        return Theme(
          data: Theme.of(context).copyWith(
            colorScheme: ColorScheme.light(
              primary: ThemeService.mainColor,
              onPrimary: Colors.white,
              onSurface: Colors.black,
            ),
            textButtonTheme: TextButtonThemeData(
              style: TextButton.styleFrom(
                primary: Colors.black,
              ),
            ),
          ),
          child: child ?? Container(),
        );
      },
      context: context,
      initialTime: _selectedTime ?? TimeOfDay(hour: 12, minute: 0),
    );
    if (value == null) return;
    setState(() {
      _selectedTime = value;
    });
    widget.onChanged?.call(_selectedDateTime);
  }

  _onPrint() {
    if (_selectedDate == null && widget.initalValue != null) {
      if (widget.initalValue is String) {
        print('trying to parse');
        var dateTime = BaseModel.parseUTC(widget.initalValue);
        if (dateTime != null) {
          _selectedDate = dateTime; //.toLocal();
          _selectedTime = TimeOfDay(
              hour: _selectedDate!.hour, minute: _selectedDate!.minute);
        }
      } else if (widget.initalValue is DateTime) {
        var dateTime = widget.initalValue; //.toLocal();
        if (dateTime != null) {
          _selectedDate = dateTime;
          _selectedTime = TimeOfDay(
              hour: _selectedDate!.hour, minute: _selectedDate!.minute);
        }
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    _onPrint();
    return widget.label == null
        ? Tooltip(
            message: widget.description ?? '',
            child: Row(
              children: [
                MaterialButton(
                    padding: EdgeInsets.all(0),
                    onPressed: _onPressed,
                    child: Text(_selectedDate != null
                        ? intl.DateFormat("dd/MM/yyyy").format(_selectedDate!)
                        : '[Select Date]')),
                MaterialButton(
                    padding: EdgeInsets.all(0),
                    onPressed: _onPressedTime,
                    child: Text(_selectedTime != null
                        ? _selectedTime!.format(context).toString()
                        : '[Select Time]'))
              ],
            ))
        : Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              SizedBox(height: 10),
              Row(
                children: [
                  Tooltip(
                      message: widget.description ?? '',
                      child: Text(
                        widget.label ?? '',
                        style: ThemeService.labelStyle,
                      )),
                  Text(
                    widget.isRequired ? '*' : '',
                    style: ThemeService.labelStyle.copyWith(color: Colors.red),
                  ),
                ],
              ),
              Row(
                children: [
                  MaterialButton(
                      padding: EdgeInsets.all(0),
                      onPressed: _onPressed,
                      child: Text(_selectedDate != null
                          ? intl.DateFormat("dd/MM/yyyy").format(_selectedDate!)
                          : '[Select Date]')),
                  MaterialButton(
                      padding: EdgeInsets.all(0),
                      onPressed: _onPressedTime,
                      child: Text(_selectedTime != null
                          ? _selectedTime!.format(context).toString()
                          : '[Select Time]')),
                ],
              ),
              SizedBox(height: 10),
              Container(height: 1, color: ThemeService.textSubColor)
            ],
          );
  }
}
