part of tecfy_core_client_package;

class SyncPage extends StatefulWidget {
  @override
  _SyncPageState createState() => _SyncPageState();
}

class _SyncPageState extends AppState<SyncPage> {
  List<dynamic>? _data;
  late ScrollController _scrollController;
  late StreamSubscription<bool> _streamSubscription;
  @override
  initState() {
    _scrollController = ScrollController();

    TecfySyncService.getUnpostedRecords().then((d) {
      _data = d;
      setState(() {});
    });
    _streamSubscription = TecfySyncService.syncWidgetStream.listen((event) {
      if (event && _data != null) return;
      TecfySyncService.getUnpostedRecords().then((d) {
        _data = d;
        setState(() {});
      });
    });
    super.initState();
  }

  @override
  void dispose() {
    _scrollController.dispose();
    _streamSubscription.cancel();
    super.dispose();
  }

  _onTab(item) async {
    if (item['type'] == 'Delete') {
      var result = await AppNavigator.confirmMessage(
          context, 'Are you sure you want to undo Delete?');
      if (result) {
        await TecfySyncService.undoDelete(item['id']);
        _data = await TecfySyncService.getUnpostedRecords();
        setState(() {});
      }
    } else {
      var result = AppNavigator.showAppDialog(context,
          (_) => EntityFormPage(item['entity'], data: {'id': item['id']}));
      print('update result ------- $result');
    }
  }

  _onTabError(item) async {
    if (item['innerError'] != null) {
      AppNavigator.showMessage(context, '', message: item['innerError']);
      // AppNavigator.showAppDialog(context,
      //     (c) => Material(child: Center(child: Text(item['innerError']))));
    }
  }

  @override
  Widget build(BuildContext context) {
    return BasePage(
      context,
      AppLocale.translate('Settings'),
      'settings',
      isMobile: LayoutService.isMobile(context),
      appBarActions: [
        TecfySyncService.getSyncIcon(context, false)
        // TecfySyncService.isSyncing
        //     ? CircularProgressIndicator()
        //     : IconButton(
        //         tooltip: trans('Sync'),
        //         icon: Icon(Icons.sync_alt),
        //         onPressed: () async {
        //           TecfySyncService.isSyncing = true;
        //           setState(() {});
        //           await TecfySyncService.pushNow();
        //           _data = await TecfySyncService.getUnpostedRecords();
        //           TecfySyncService.isSyncing = false;
        //           setState(() {});
        //         })
      ],
      child: _data == null
          ? Center(child: CircularProgressIndicator())
          : Scrollbar(
              isAlwaysShown: true,
              showTrackOnHover: true,
              controller: _scrollController,
              child: SingleChildScrollView(
                controller: _scrollController,
                child: DataTable(
                  showCheckboxColumn: false,
                  sortColumnIndex: 1,
                  sortAscending: false,
                  columns: [
                    DataColumn(label: Text(trans('Entity')), numeric: false),
                    DataColumn(label: Text(trans('Type')), numeric: false),
                    DataColumn(label: Text(trans('Time')), numeric: false),
                    DataColumn(label: Text(trans('Message')), numeric: false)
                  ],
                  rows: _data
                          ?.map<DataRow>((d) => DataRow(
                                cells: [
                                  DataCell(
                                      Text(EntityService.getEntity(d['entity'])
                                              ?.nameAr ??
                                          ''),
                                      onTap: () => _onTab(d)),
                                  DataCell(Text(d['type'].toString()),
                                      onTap: () => _onTab(d)),
                                  DataCell(
                                      Text(DateTime.fromMillisecondsSinceEpoch(
                                              d['timeStamp'] ?? 0)
                                          .toString()),
                                      onTap: () => _onTab(d)),
                                  DataCell(Text(d['postMessage'] ?? ''),
                                      onTap: () => _onTabError(d)),
                                ],
                              ))
                          .toList() ??
                      [],
                ),
              ),
            ),
    );
  }
}
