part of tecfy_core_client_package;

class SettingPage extends StatefulWidget {
  @override
  _SettingPageState createState() => _SettingPageState();
}

class _SettingPageState extends AppState<SettingPage> {
  int _unPostedCount = 0;
  bool _syncing = false;

  @override
  void initState() {
    TecfySyncService.getUnpostedCount().then((r) {
      _unPostedCount = r;
      setState(() {});
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return BasePage(
      context,
      AppLocale.translate('Settings'),
      'settings',
      isMobile: LayoutService.isMobile(context),
      child: SingleChildScrollView(
        child: Column(
          children: <Widget>[
            SizedBox(height: 15.0),
            _getItem(
                'Language',
                DropdownButtonHideUnderline(
                    child: DropdownButton<Language>(
                  value: LocalConfigService.language,
                  items: [
                    DropdownMenuItem(
                        child: Text('عربي'), value: Language.arabic),
                    DropdownMenuItem(
                        child: Text('English'), value: Language.english)
                  ],
                  onChanged: (v) async {
                    LocalConfigService.language = v ?? Language.arabic;
                    setState(() {});
                  },
                ))),
            SizedBox(height: 15.0),
            _getItem(
                'Connection Status',
                Row(
                  children: <Widget>[
                    Text(trans('Status')),
                    Text(':'),
                    SizedBox(width: 15.0),
                    ConnectionService.getConnectionIconStream(),
                  ],
                )),
            SizedBox(height: 15.0),
            !PlatformService.isSupportLocalDBSupported
                ? Container()
                : _getItem(
                    'Operation Mode',
                    Row(
                        mainAxisAlignment: MainAxisAlignment.spaceBetween,
                        children: [
                          DropdownButtonHideUnderline(
                              child: DropdownButton<OperationMode>(
                            value: LocalConfigService.operationMode,
                            items: [
                              DropdownMenuItem(
                                  child: Text(trans('Online Mode')),
                                  value: OperationMode.online),
                              DropdownMenuItem(
                                  child: Text(trans(
                                      'Offline: Cache Required Data Only')),
                                  value: OperationMode.offline_cache_required),
                              DropdownMenuItem(
                                  child: Text(
                                      trans('Offline: Cache Last 3 months')),
                                  value: OperationMode.offline_cache_3months),
                              DropdownMenuItem(
                                  child: Text(
                                      trans('Offline: Cache Last 6 months')),
                                  value: OperationMode.offline_cache_6months),
                              DropdownMenuItem(
                                  child: Text(
                                      trans('Offline: Cache Last 12 months')),
                                  value: OperationMode.offline_cache_12months),
                              DropdownMenuItem(
                                  child: Text(trans('Offline: Cache All Data')),
                                  value: OperationMode.offline_cache_all),
                            ],
                            onChanged: (v) async {
                              if (LocalConfigService.operationMode !=
                                      OperationMode.online &&
                                  v == OperationMode.online) {
                                // make sure nothing in cache
                                try {
                                  int pending =
                                      await TecfySyncService.getUnpostedCount();
                                  if (pending > 0) {
                                    var result = await AppNavigator.confirmMessage(
                                        context,
                                        'You have $pending local items not posted to server!!!!',
                                        message: AppLocale.translate(
                                            'Are you sure you want to logout?'));
                                    if (!result) return;
                                  }
                                } catch (e) {}
                              } else if (LocalConfigService.operationMode ==
                                      OperationMode.online &&
                                  v != OperationMode.online) {
                                // re-create database
                                AppNavigator.showLoading(context);
                                await DatabaseService.deleteDatabaseFile();
                                await EntityService.initEntities();
                                await DatabaseService.init();
                                AppNavigator.pop(context);
                              }
                              LocalConfigService.operationMode =
                                  v ?? OperationMode.online;
                              setState(() {});
                            },
                          )),
                          LocalConfigService.operationMode !=
                                  OperationMode.online
                              ? IconButton(
                                  tooltip: trans('Show'),
                                  icon: Icon(Icons.remove_red_eye_outlined),
                                  onPressed: () async {
                                    await AppNavigator.navigateTo(
                                        context, () => LocalRecordsPage());
                                    _unPostedCount = await TecfySyncService
                                        .getUnpostedCount();
                                    if (mounted)
                                      setState(() => _syncing = false);
                                  })
                              : Container()
                        ])),
            SizedBox(height: 15.0),
            ConnectionService.isOnlineMode
                ? Container()
                : _getItem(
                    'Unposted Items',
                    Row(
                      children: <Widget>[
                        Text(trans('Unposted Items')),
                        // Text(':'),
                        // SizedBox(width: 15.0),
                        // Text(_unPostedCount.toString()),
                        SizedBox(width: 30.0),
                        TecfySyncService.getSyncIcon(context, false),
                        // _syncing ? CircularProgressIndicator() : Container(),
                        // _syncing || ConnectionService.isOffline
                        //     ? Container()
                        //     : IconButton(
                        //         tooltip: trans('Push Now'),
                        //         icon: Icon(Icons.sync_alt),
                        //         onPressed: () async {
                        //           setState(() => _syncing = true);
                        //           await TecfySyncService.pushNow();
                        //           _unPostedCount =
                        //               await TecfySyncService.getUnpostedCount();
                        //           setState(() => _syncing = false);
                        //         }),
                        Expanded(child: Container()),
                        IconButton(
                            tooltip: trans('Show'),
                            icon: Icon(Icons.remove_red_eye_outlined),
                            onPressed: () async {
                              await AppNavigator.navigateTo(
                                  context, () => SyncPage());
                              _unPostedCount =
                                  await TecfySyncService.getUnpostedCount();
                              setState(() => _syncing = false);
                            })
                      ],
                    )),
            SizedBox(height: 15.0),
            _getItem(
                'Account Settings',
                Row(
                  children: <Widget>[
                    Text(trans('Name')),
                    Text(':'),
                    SizedBox(width: 15.0),
                    Text(LocalConfigService.user?.fullName ?? trans("Guest")),
                    SizedBox(width: 30.0),
                    IconButton(
                        tooltip: trans('Update Info'),
                        icon: Icon(Icons.edit),
                        onPressed: () async {
                          await AppNavigator.navigateTo(
                              context, () => AccountPage());
                        })
                  ],
                )),
            SizedBox(height: 15.0),
            _getItem(
                'Enable Expert Mode',
                Switch(
                    value: ConfigService.getValueBool('ExpertMode'),
                    onChanged: (v) {
                      ConfigService.setValueBool('ExpertMode', v);
                      setState(() {});
                    }))
          ],
        ),
      ),
    );
  }

  Widget _getItem(String title, Widget child) {
    return Card(
      margin: EdgeInsets.symmetric(horizontal: 15.0),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.stretch,
        children: <Widget>[
          Container(
            height: 45.0,
            color: ThemeService.backgroundSecondColor,
            child: Padding(
                padding: EdgeInsets.symmetric(horizontal: 10),
                child: Align(
                    alignment: LocalConfigService.isArabic
                        ? Alignment.centerRight
                        : Alignment.centerLeft,
                    child: Text(
                      AppLocale.translate(title),
                      style: Theme.of(context).textTheme.caption,
                    ))),
          ),
          //item('Account Information', ()=>AccountPage())
          Container(
              height: 70.0,
              color: Colors.white,
              margin: EdgeInsets.symmetric(horizontal: 10),
              child: Align(
                  alignment: LocalConfigService.isArabic
                      ? Alignment.centerRight
                      : Alignment.centerLeft,
                  child: child))
        ],
      ),
    );
  }

  Widget _getBoarder(widget) {
    return Container(
        height: 60.0,
        decoration: BoxDecoration(
            border: Border.all(color: ThemeService.mainColor),
            borderRadius: BorderRadius.all(Radius.circular(2.0))),
        child: widget);
  }

  Widget _divider() {
    return Padding(
        padding: EdgeInsets.symmetric(horizontal: 10.0),
        child:
            Container(height: 1.0, color: ThemeService.backgroundSecondColor));
  }

  Widget _item(title, creator) {
    return GestureDetector(
        onTap: creator == null
            ? null
            : () => AppNavigator.navigateTo(context, creator),
        child: Container(
          height: 70.0,
          color: Colors.white,
          child: Align(
              alignment: Alignment.centerLeft,
              child: Row(
                children: <Widget>[
                  SizedBox(width: 10.0),
                  Expanded(
                      child: Text(
                    title,
                    style: Theme.of(context).textTheme.subtitle1,
                  )),
                  SizedBox(width: 10.0),
                  Icon(
                    Icons.arrow_right,
                    color: Theme.of(context).textTheme.subtitle1?.color ??
                        Colors.black,
                  ),
                  SizedBox(width: 10.0),
                ],
              )),
        ));
  }

  Future<String> _getAppVersion() async {
    // PackageInfo packageInfo = await PackageInfo.fromPlatform();
    return ''; //packageInfo.version;
  }
}
